#include "GSAT/GSAT.hpp"
#include "GSATThread.hpp"
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

using namespace std;

void test() {
    /*ControlBandit cb;
    fillAndHeuristic fah;
    while(!cb.queueIsEmpty()) {
        fah = cb.next();
        printf("%d %d\n", fah.fill, fah.heuristic);
    }*/

    srand(getpid());
    double ref = 0;
    double avg = 0;
    int nb = 0;

    int rdm;
    for(int i = 0; i < 100; i++) {
        rdm = rand();
        avg = avg + (1 / (nb + 1.0)) * (rdm - avg);
        nb++;
        ref += rdm;
    }
    printf("ref : %f, ite : %f\n", ref/100.0, avg);

    exit(0);
}

void help(char* prog) {
    fprintf(stderr, "usage: %s -i file.cnf [-t int] [-s] [-e double]\n\t-t Number of threads to use\n\t-s Silent mode\n\t-e Use epsilon greedy method with the value (0 <= value <= 1)\n", prog);
    exit(1);
}

int main(int argc, char* argv[]) {

    //test();

    extern char * optarg; 
    int nbThread = DEFAULT_NB_THREAD;
    bool verbose = true;
    int opt;
    bool optI = false;
    bool optE = false;
    double epsilon = 0.1;

    char** gsatArg = (char**) malloc(sizeof(char*) * 3);
    gsatArg[0] = (char*) malloc(sizeof(char) * strlen(argv[0]) + 1);
    memset(gsatArg[0], 0, strlen(argv[0]) + 1);
    strncpy(gsatArg[0], argv[0], strlen(argv[0]));

    while((opt = getopt(argc, argv, "si:t:e:")) != -1) {
        switch(opt) {
            case 'i':
                optI = true;
                gsatArg[1] = (char*) malloc(sizeof(char) * 3);
                memset(gsatArg[1], 0, 3);
                strncpy(gsatArg[1], "-i", 2);
                gsatArg[2] = (char*) malloc(sizeof(char) * strlen(optarg) + 1);
                memset(gsatArg[2], 0, strlen(optarg) + 1);
                strncpy(gsatArg[2], optarg, strlen(optarg));
                break;
            case 't':
                nbThread = atoi(optarg);
                break;
            case 's':
                verbose = false;
                break;
            case 'e':
                optE = true;
                epsilon = strtod(optarg, NULL);
                break;
            case '?':
                help(argv[0]); 
        }
    }

    if(!optI) {
        help(argv[0]);
    }
  
    GSATThread* gsat = new GSATThread(nbThread, 3, gsatArg);
    if(optE) {
        gsat->useEpsilonMethod(epsilon);
    }
    printf("c nbThreads: %d\n", nbThread);
    printf("c nbVariables: %d\n", gsat->getNbVariables());
    printf("c nbClauses: %d\n", gsat->getNbClauses());
    bool result = gsat->solve(verbose);
    if(!verbose) {
        printf("s %s\n", result?"SATISFIABLE":"NOT FOUND");
    }
    

}